package de.mathema.spring.impl;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.MailSender;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import de.mathema.spring.api.Subscriber;
import de.mathema.spring.api.Subscription;

@Component("de.mathema.spring.api.Subscription")
@Transactional(propagation=Propagation.REQUIRED)
public class SubscriptionImpl implements Subscription {
	@PersistenceContext
	private EntityManager em;
	@Autowired
	private MailSender mailSender;
	
	public void setEm(EntityManager em) {
		this.em = em;
	}

	public void setMailSender(MailSender mailSender) {
		this.mailSender = mailSender;
	}

	@Override
	public Subscriber confirm(long id) {
		Subscriber s = em.find(Subscriber.class, id);
		if(s!= null){
			s.setConfirmed(true);
		}
		return s;
	}

	@Override
	public Subscriber create(String nickName, String email) {
		Subscriber s = new Subscriber(nickName, email);
		em.persist(s);
		SimpleMailMessage message = new SimpleMailMessage();
		message.setFrom("werner@mathema.de");
		message.setSubject("Willkommen im Club");
		message.setTo(email);
		message.setText("Hallo " + nickName + "\n\nDu hast gerade dich fr unseren Newsletter angemeldet. Du musst aber noch besttigen. Click einfach hier:\n\n" + 
					"http://localhost:8080/TechTrack/SubscriptionServlet?id=" + s.getId());
		mailSender.send(message);
		System.out.println(message);
		return s;
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Subscriber> listAll(boolean confirmed) {
		String queryString = "select s from Subscriber s " + (confirmed?"WHERE s.confirmed=true":"");
		Query q = em.createQuery(queryString);
		
		return q.getResultList();
	}

	@Override
	public int sendEmailToAllConfirmed(String subject, String text) {
		List<Subscriber> all = listAll(true);
		SimpleMailMessage message = new SimpleMailMessage();
		message.setFrom("werner@mathema.de");
		int i = 0;
		for (Subscriber subscriber : all) {
			message.setTo(subscriber.getEmail());
			message.setSubject(subject.replaceAll("%NICK%", subscriber.getNickName()));
			message.setText(text.replaceAll("%NICK%", subscriber.getNickName()));
			mailSender.send(message);
			i++;
		}
		return i;
	}

}
